@@ -9,6 +9,10 @@ import boto3
 from botocore.exceptions import ClientError
 from django.utils.text import slugify
 
+from PIL import Image, ImageOps
+from io import BytesIO
+from multiprocessing.pool import ThreadPool
+
 # import settings
 from tasq_cli.db import UploadsDatabase
 # from utils import timestamp_for_file_name
@@ -26,34 +30,42 @@ def upload_file(dataset_name, db, settings, client, file_name_with_path, leaf_pa
     logger.debug(f'Checking if file needs to be uploaded ...')
 
     # before upload check if file exists on server under this dataset
-    with open(f'{file_path}/{file_name}', 'rb') as fd:
-        data = fd.read()
-        md5 = hashlib.md5(data)
+    with open(f'{file_path}/{file_name}', 'rb') as infile:
+        image_wrapper = BytesIO()
+        # if file is png, jpg, or jpeg - rotate
+        if ext in ['png', 'jpg', 'jpeg']:
+            image = Image.open(infile)
+            transposed_image = ImageOps.exif_transpose(image)
+            transposed_image.save(image_wrapper, ('JPEG' if ext == 'jpg' else ext))
+        else:
+            image_wrapper = infile
+
+        md5 = hashlib.md5(image_wrapper.getvalue() if 'getvalue' in dir(image_wrapper) else image_wrapper.read())
+        image_wrapper.seek(0)
         content_md5 = base64.b64encode(md5.digest()).decode('utf-8')
         md5_hash = md5.hexdigest()
 
-    logger.debug(f'File hash: {md5_hash}')
-    uploaded_entry = db.get_upload(dataset_name, md5_hash)
+        logger.debug(f'File hash: {md5_hash}')
+        uploaded_entry = db.get_upload(dataset_name, md5_hash)
 
-    if uploaded_entry:
-        logger.debug(f'\tFile with hash {md5_hash} is already uploaded to dataset {dataset_name}.')
-        return uploaded_entry
+        if uploaded_entry:
+            logger.debug(f'\tFile with hash {md5_hash} is already uploaded to dataset {dataset_name}.')
+            return uploaded_entry
 
-    logger.info(f'\tUploading to {settings.BUCKET} as {object_name} ...')
-    try:
-        upload_entry = {
-                'bucket': settings.BUCKET,
-                'dataset_name': dataset_name,
-                'file_name': f'{relative_file_path}/{file_name}',
-                'object_name': object_name,
-                'md5_hash': md5_hash,
-                'url': f'https://{settings.BUCKET}.s3.amazonaws.com/{object_name}',
-                'cdn_url': settings.CDN_URL.format(object_name=object_name),
-            }
-        if not settings.dry:
-            with open(f'{file_path}/{file_name}', 'rb') as fd:
+        logger.info(f'\tUploading to {settings.BUCKET} as {object_name} ...')
+        try:
+            upload_entry = {
+                    'bucket': settings.BUCKET,
+                    'dataset_name': dataset_name,
+                    'file_name': f'{relative_file_path}/{file_name}',
+                    'object_name': object_name,
+                    'md5_hash': md5_hash,
+                    'url': f'https://{settings.BUCKET}.s3.amazonaws.com/{object_name}',
+                    'cdn_url': settings.CDN_URL.format(object_name=object_name),
+                }
+            if not settings.dry:
                 response = client.put_object(
-                    Body=fd,
+                    Body=image_wrapper,
                     Bucket=settings.BUCKET,
                     Key=object_name,
                     ContentMD5=content_md5,
@@ -64,30 +76,30 @@ def upload_file(dataset_name, db, settings, client, file_name_with_path, leaf_pa
                     # NOTE
                     # consider that sync might work faster than put_object
                 )
-            if response['ResponseMetadata']['HTTPStatusCode'] == 200 and response['ETag'] == f'"{md5_hash}"':
-                tag_response = client.put_object_tagging(
-                    Bucket=settings.BUCKET,
-                    Key=object_name,
-                    Tagging={
-                        'TagSet': [
-                            {
-                                'Key': 'original_name',
-                                'Value': slugify(file_name)
-                            },
-                        ]
-                    }
-                )
-                
-                return upload_entry
+                if response['ResponseMetadata']['HTTPStatusCode'] == 200 and response['ETag'] == f'"{md5_hash}"':
+                    tag_response = client.put_object_tagging(
+                        Bucket=settings.BUCKET,
+                        Key=object_name,
+                        Tagging={
+                            'TagSet': [
+                                {
+                                    'Key': 'original_name',
+                                    'Value': slugify(file_name)
+                                },
+                            ]
+                        }
+                    )
+                    
+                    return upload_entry
+                else:
+                    logger.error(response)
+                    return None
             else:
-                logger.error(response)
-                return None
-        else:
-            return upload_entry
-    except ClientError as e:
-        logger.error(e)
+                return upload_entry
+        except ClientError as e:
+            logger.error(e)
+            return None
         return None
-    return None
 
 
 def get_files(dir_path, excluded_extensions):
@@ -129,8 +141,6 @@ def do_upload(dataset_name, search_path, exclude, settings):
 
     logger.info(f'Using access key {settings.ACCESS_KEY[:3]}*********{settings.ACCESS_KEY[-3:]} and secret {settings.SECRET_KEY[:3]}*********{settings.SECRET_KEY[-3:]}.')
 
-    db = UploadsDatabase()
-
     leaf_path = os.path.basename(os.path.normpath(search_path))
     added_files = []
 
@@ -138,8 +148,9 @@ def do_upload(dataset_name, search_path, exclude, settings):
     num_files = len(files)
     logger.info(f'Found {num_files} files.')
 
-    for i, file_name in enumerate(files):
-        logger.info(f'Uploading file [{i + 1}/{num_files}] {file_name} {settings.BUCKET}')
+    def single_upload(file_name):
+        logger.info(f'Uploading file {file_name} {settings.BUCKET}')
+        db = UploadsDatabase()
         upload_entry = upload_file(dataset_name, db, settings, client, file_name, leaf_path) 
         if not upload_entry:
             db.close()
@@ -148,7 +159,10 @@ def do_upload(dataset_name, search_path, exclude, settings):
             added_files.append(upload_entry)
             if not settings.dry:
                 db.insert_upload(**upload_entry)
-    db.close()
+        db.close()
+
+    pool = ThreadPool(processes=10)
+    pool.map(single_upload, files)
 
     # write out image urls to csv
     os.makedirs("./out/", exist_ok=True)
@@ -189,6 +203,8 @@ def create_csv_from_currently_uploaded_files(dataset_name, file_name, settings):
             entry[row_values['cdn_url']],
             json.dumps({'file_name': entry[row_values['file_name']]}),  # object
         ] for entry in entries]
+    if not os.path.exists('./out/'):
+        os.makedirs('./out')
     with open(f'./out/{dataset_name}.csv', 'w') as csvfile:
         csvwriter = csv.writer(csvfile, dialect='unix')
         csvwriter.writerows(csv_rows)
